@@ -23,6 +23,7 @@
 
 import json
 import re
+import ast
 import sys
 import urllib.parse
 from collections import defaultdict
@@ -320,7 +321,7 @@ def create_charts(df):
         # Create a new dataframe with the split countries data
         new_df = split_df.to_frame().join(df[["name"]]).reset_index(drop=True)
 
-        # Group the data by country and count the number of users for each country
+        # Group the data by hashtags and count the number of users for each hashtag
         grouped = (
             new_df.groupby("hashtags")["name"].count().sort_values(ascending=False)
         )
@@ -333,7 +334,7 @@ def create_charts(df):
         fig, ax = plt.subplots(figsize=(20, 20))
         ax = sns.barplot(x=grouped.index, y=grouped.values)
 
-        font = fm.FontProperties(family="Arial", size=8)
+        font = fm.FontProperties(family="Arial", size=12)
         # Add the count labels to the bars
         for i, v in enumerate(grouped.values):
             ax.text(
@@ -402,7 +403,7 @@ def create_charts(df):
             x_pos,
             [create_data.get(k, 0) for k in keys],
             bar_width,
-            color="b",
+            color="g",
             label="Create",
         )
         for i, bar in enumerate(bar1):
@@ -422,7 +423,7 @@ def create_charts(df):
             x_pos + bar_width,
             [modify_data.get(k, 0) for k in keys],
             bar_width,
-            color="r",
+            color="b",
             label="Modify",
         )
         for i, bar in enumerate(bar2):
@@ -464,3 +465,14 @@ def create_profile_link(name):
     # Create URL string
     url = f"https://www.openstreetmap.org/user/{encoded_name}"
     return url
+
+# Define a function to sum up the tag values across all rows
+def sum_tags(tags_list):
+    tag_counts = {}
+    for tags_dict in tags_list:
+        tags_dict=ast.literal_eval(tags_dict)
+        for tag, count in tags_dict.items():
+            if tag not in tag_counts:
+                tag_counts[tag] = 0
+            tag_counts[tag] += count
+    return tag_counts
\ No newline at end of file
